;;; gnus-url.el    makes URLs and filenames hot-links in GNUS mail msgs
;;;
;;; Arup Mukherjee 11/93


;;; This file provides hooks to use hot-url in gnus,
;;; making hot-links out of URLs, message-ids, and/or absolute filenames.
;;;
;;; To follow one of these links, move point to it and press return,
;;; or, in lemacs, just click the middle mouse button on the link.
;;;
;;; To use this, just add the following to your .emacs:
;;;
;;;  (setq gnus-startup-hook (list (function (lambda () (require 'gnus-url)))))
;;;  [Note: for older versions of gnus, use gnus-Startup-hook instead]
;;;
;;; ... and set hot-url-fetch-url-function appropriately if you want to use
;;; Mosaic (instead of emacs-based w3-mode) to view fetched URLs.
;;; (See hot-url.el for more details).
;;;
;;; The following packages are used to provide this feature:
;;;     hot-link.el             support for creating & following hot links
;;;     hot-url.el              turns urls and filenames into hot links
;;;     w3-fetch-url.el         function for using w3 to fetch a URL
;;;     mosaic-fetch-url.el     function for using NCSA Mosaic to fetch a URL

(require 'hot-url)

(if (featurep 'gnus-url)
    (error "gnus-url already loaded"))

;; These variables are obsolete.  Use hot-url-default-hot-types instead.
;; (defvar gnus-hot-urls t
;;   "If non-nil, make URLs in articles into hot-links.")
;; 
;; (defvar gnus-hot-msgids t
;;  "If non-nil, make message-ids (in angle brackets) in articles
;; into hot-links.")
;; 
;; (defvar gnus-hot-filenames t
;;  "If non-nil, make absolute filenames in articles into hot-links.")

(defvar gnus-hot-url-map nil
  "keymap for hot-url extents in gnus")

;;;  default key bindings -- hit return to follow the link the cursor is on.
;;;                          in lemacs, press button2 to follow a link.

(if (fboundp 'extent-property) ;; lemacs 19.9 & later
    (progn
      (setq gnus-hot-url-map (make-sparse-keymap))
      (define-key gnus-hot-url-map 'button2 'hot-link-follow-mouse)
      (define-key gnus-hot-url-map "\C-m" 'hot-link-follow-point))
  (setq gnus-hot-url-map nil)
  (if (string-match "Lucid" emacs-version)
      (hot-link-bind-maybe-follow-mouse gnus-Article-mode-map 'button2 
					(lookup-key gnus-Article-mode-map
						    'button2)))
  (hot-link-bind-maybe-follow-point gnus-Article-mode-map "\C-m"
				    (lookup-key gnus-Article-mode-map "\C-m")))
  

(if (boundp 'gnus-Article-prepare-hook)
    ;; older versions of gnus
    (setq gnus-Article-prepare-hook
	  (if (listp gnus-Article-prepare-hook)
	      (append gnus-Article-prepare-hook '(gnus-make-hot-urls))
	    (list gnus-Article-prepare-hook 'gnus-make-hot-urls)))
  
  ;; more recent versions
  (setq gnus-article-prepare-hook  
	(if (listp gnus-article-prepare-hook)
	    (append gnus-article-prepare-hook '(gnus-make-hot-urls))
	  (list gnus-article-prepare-hook 'gnus-make-hot-urls))))
  

(defun gnus-make-hot-urls ()
"Make URLs, message-ids, and/or absolute filenames in the current buffer 
into hot-links.  
Intended for use in gnus-article-prepare-hook."
  (hot-url-make-all-hot (point-min) (point-max) gnus-hot-url-map))

(provide 'gnus-url)

