;;; mh-e-url.el    makes URLs and filenames hot-links in mh-e mail msgs

;;; This file provides hooks to use hot-url in mh-e,
;;; making hot-links out of URLs, message-ids, and/or absolute filenames.
;;;
;;; To follow one of these links, move point to it and press return,
;;; or, in lemacs, just click the middle mouse button on the link.
;;;
;;; To use this, put something like this
;;;   (setq mh-folder-mode-hook
;;;         (list '(lambda () (require 'mh-e-url))))
;;; in your .emacs file, and set hot-url-fetch-url-function 
;;; appropriately if you want to use Mosaic (rather than the emacs-based 
;;; w3-mode) to display the fetched URLs (see hot-url.el).
;;;
;;; The following packages are used to provide this feature:
;;;     hot-link.el             support for creating & following hot links
;;;     hot-url.el              turns urls and filenames into hot links
;;;     w3-fetch-url.el         function for using w3 to fetch a URL
;;;     mosaic-fetch-url.el     function for using NCSA Mosaic to fetch a URL
;;;

(require 'hot-url)
(require 'mh-e)              ; sorry 'bout this. if mh-e had reasonable hooks,
		   	     ; it wouldn't be necessary

(if (featurep 'mh-e-url)
    (error "mh-e-url already loaded"))

;; These variables are obsolete.  Use hot-url-default-hot-types instead.
;; (defvar mh-e-hot-urls t
;;   "If non-nil, make URLs in mail messages into hot-links.")
;; 
;; (defvar mh-e-hot-msgids t
;;  "If non-nil, make message-ids (in angle brackets) in mail messages
;; into hot-links.")
;; 
;; (defvar mh-e-hot-filenames t
;;  "If non-nil, make absolute filenames in mail messages into hot-links.")

(defvar mh-hot-link-map nil
  "keymap for hot-url extents in gnus")

(if (fboundp 'extent-property) ;; lemacs 19.9 & later
    (progn
      (setq mh-hot-link-map (make-sparse-keymap))
      (define-key mh-hot-link-map 'button2 'hot-link-follow-mouse)
      (define-key mh-hot-link-map "\C-m" 'hot-link-follow-point))
  (setq mh-hot-link-map nil)
  (setq mh-display-letter-map (make-sparse-keymap))
  (if (string-match "XEmacs\\|Lucid" emacs-version)
      (hot-link-bind-maybe-follow-mouse mh-display-letter-map
					'button2
					(global-key-binding 'button2)))
  (hot-link-bind-maybe-follow-point mh-display-letter-map
				    "\C-m"
				    (global-key-binding "\C-m")))

;;; save the old value of mh-display-msg, and replace it with a wrapper 
;;; which calls the original mh-display-msg and then scans the buffer
;;; for URLs, etc., and sets up the appropriate keymap if necessary.
;;; (should probably use a full-fledged major mode here, but none of
;;; this keymap nonsense is necessary under lemacs 19.9 and later)
(fset 'mh-e-url::bkp-display-msg (symbol-function 'mh-display-msg))

(defun mh-display-msg (msg-num folder)
  (mh-e-url::bkp-display-msg msg-num folder)
  (let*
      ((body-begin (progn (goto-char (point-min))
			  (search-forward "\n\n" nil 'go-to-end)
			  (point)))
       (body-end (point-max)))
    (hot-url-make-all-hot body-begin body-end mh-hot-link-map))
  (if (not (fboundp 'extent-property)) ;; lemacs 19.8 & earlier, FSF18 & 19
      (use-local-map mh-display-letter-map)))

(provide 'mh-e-url)
