;; 
;; Performs options negotiation with a telnetd process 
;;
;; Arup Mukherjee 11/93

(setq telnet-command-escape "\377")
(setq telnet-command-format "\377..")
(setq telnet-dont-use       "\377\376")
(setq telnet-please-use     "\377\375")
(setq telnet-wont-use       "\377\374")
(setq telnet-will-use       "\377\373")
(setq telnet-encrypt-option "&")        ;; specific to CMU-CS
(setq telnet-binary-option  "\000")
;;; request binary transmission so we can send 8-bit characters.
(setq telnet-initial-requests
      (concat
        telnet-will-use telnet-binary-option
        telnet-please-use telnet-binary-option))

(defun telnet-do-negotiation (proc buffer start-point)
  (save-excursion 
    (set-buffer buffer)
    (if (featurep 'mule)
	(set-buffer-multibyte nil))
    (goto-char start-point)

    ;; wait for output
    (message "waiting...")
    (while (not (re-search-forward telnet-command-escape nil t))
      (accept-process-output)
      (goto-char start-point))
    (goto-char start-point)
    (message "proceeding")

    (process-send-string proc telnet-initial-requests)
    (while (not (re-search-forward "
" nil t))
      (accept-process-output)
      (if (looking-at telnet-command-format)
	  (let ((response
		 (telnet-make-response 
		  (buffer-substring (point) (+ (point) 3)))))
	    (if response (process-send-string proc response))
	    (forward-char 2)))
      (accept-process-output)
      (if (not (looking-at telnet-command-escape))
	  (if (re-search-forward telnet-command-escape nil t)
	      (forward-char -1))))))

(defun telnet-make-response (s)
  (let ((request (substring s 0 2))
	(option  (substring s 2 3)))
    (cond ((string= request telnet-please-use)        ; srvr: pls use X
	   (concat telnet-wont-use option))           ; clnt: sorry, no X
	  ((string= request telnet-will-use)          ; srvr: will use X
	   (if (string= option telnet-encrypt-option) ; is X encryption?
	       (concat telnet-dont-use option)        ; clnt: no encryption
	     nil))                                    ; clnt: as you wish
	  ((string= request telnet-dont-use)          ; srvr: don't use X
	   (concat telnet-wont-use option)))))        ; clnt: won't use X

(provide 'telnet-conn)
